package com.agilecontrol.nea.core.control.web;

import com.agilecontrol.nea.core.control.web.SessionValueCacheManager;
import com.agilecontrol.nea.core.control.web.WebUtils;
import com.agilecontrol.nea.core.control.web.filter.dto.HttpUtils;
import com.agilecontrol.nea.core.control.web.filter.dto.ResponseMsg;
import com.agilecontrol.nea.core.control.web.filter.dto.UserCenterLogin;
import com.agilecontrol.nea.core.util.AdSqlUtils;
import com.agilecontrol.nea.core.util.ConfigValues;
import com.agilecontrol.nea.core.util.CookieKeys;
import com.agilecontrol.nea.core.util.HttpComponentsUtil;
import com.agilecontrol.nea.core.util.JsonUtil;
import com.agilecontrol.nea.core.util.RedisUtils;
import com.agilecontrol.nea.util.NDSRuntimeException;
import com.agilecontrol.nea.util.Tools;
import com.agilecontrol.nea.util.Validator;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.commons.lang.StringUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StopWatch;
import redis.clients.jedis.Jedis;

public class SessionUtils {

   private static Logger logger = LoggerFactory.getLogger(SessionUtils.class);
   public static final String LOCAL_TOKEN_PREFIX = "PT-";


   public static String getClientAddress(HttpServletRequest request) {
      return HttpComponentsUtil.getClientAddress(request);
   }

   public static String getSessionKey(HttpServletRequest request) {
      String sessionkey = (String)request.getAttribute("token");
      if(StringUtils.isBlank(sessionkey)) {
         sessionkey = request.getHeader("Token");
      }

      if(StringUtils.isBlank(sessionkey)) {
         sessionkey = request.getHeader("token");
      }

      if(StringUtils.isBlank(sessionkey)) {
         sessionkey = request.getHeader("TOKEN");
      }

      if(StringUtils.isBlank(sessionkey)) {
         sessionkey = request.getParameter("token");
      }

      if(StringUtils.isBlank(sessionkey)) {
         sessionkey = request.getParameter("Token");
      }

      if(StringUtils.isBlank(sessionkey)) {
         Cookie cookie = CookieKeys.getCookieObj(request, "SSOTOKEN");
         if(cookie != null) {
            sessionkey = cookie.getValue();
         }

         if(StringUtils.isBlank(sessionkey)) {
            cookie = CookieKeys.getCookieObj(request, "sso-token");
            if(cookie != null) {
               sessionkey = cookie.getValue();
            }
         }
      }

      return sessionkey;
   }

   public static String getUserCenterTokenURL(HttpServletRequest request) {
      String tokenUrl = ConfigValues.USERCENTER_TOKEN_URL;
      if(Validator.isNull(tokenUrl)) {
         try {
            tokenUrl = AdSqlUtils.getADSQL("usercenter_check_token", false);
         } catch (Exception var4) {
            logger.error(var4.getMessage(), var4);
         }
      }

      if(Validator.isNull(tokenUrl)) {
         throw new NDSRuntimeException("请维护系统变量ad_param#usercenter.token.url， 如/transfer-web/user/token");
      } else {
         if(tokenUrl.startsWith("/")) {
            if(request == null) {
               throw new NDSRuntimeException("内部错误：用户中心地址为短地址模式，且request变量为空");
            }

            int port = request.getServerPort();
            String portStr = port <= 0?"":":" + port;
            tokenUrl = request.getScheme() + "://" + request.getServerName() + portStr + tokenUrl;
         }

         return tokenUrl;
      }
   }

   public static long clearRedisTokenSession(String token) {
      Jedis jedis = WebUtils.getJedis();
      Throwable var2 = null;

      long var3;
      try {
         var3 = RedisUtils.delKeyPattern("portal:token:" + token + ":*", jedis);
      } catch (Throwable var13) {
         var2 = var13;
         throw var13;
      } finally {
         if(jedis != null) {
            if(var2 != null) {
               try {
                  jedis.close();
               } catch (Throwable var12) {
                  var2.addSuppressed(var12);
               }
            } else {
               jedis.close();
            }
         }

      }

      return var3;
   }

   public static boolean isFromUserCenter(String token) {
      return token == null?false:!token.startsWith("PT-");
   }

   public static JSONObject logoutUserCenter(String token, HttpServletRequest request) {
      HashMap map = new HashMap();
      map.put("token", token);
      map.put("action", "logout");
      String checkTokenUrl = null;
      checkTokenUrl = getUserCenterTokenURL(request);
      String url = HttpUtils.buildGetParam(checkTokenUrl, map);
      JSONObject result = HttpUtils.sendGet(url);
      return result;
   }

   public static void logout(HttpServletRequest request, HttpServletResponse response) {
      int port = request.getServerPort();
      HttpSession session = request.getSession();
      String domain = CookieKeys.getDomain(request);
      Cookie passwordCookie = new Cookie("NEAPASSWORD" + port, "");
      if(Validator.isNotNull(domain)) {
         passwordCookie.setDomain(domain);
      }

      passwordCookie.setMaxAge(0);
      passwordCookie.setPath("/");
      CookieKeys.addCookie(request, response, passwordCookie);
      Cookie tokenCookie = new Cookie("token" + port, "");
      tokenCookie.setMaxAge(0);
      tokenCookie.setPath("/");
      CookieKeys.addCookie(request, response, tokenCookie);
      String token = request.getSession().getId();
      removeTokenCookie(request, response);
      boolean isRequestFromUserCenter = Tools.getBoolean(request.getParameter("usercenter"), false);
      if(!isRequestFromUserCenter && isFromUserCenter(token)) {
         logoutUserCenter(token, request);
      }

      try {
         session.invalidate();
      } catch (Exception var10) {
         logger.error("Fail to invalidate session:" + token, var10);
      }

   }

   public static UserCenterLogin getUserCenterLoginInfo(String token, HttpServletRequest request) {
      HashMap map = new HashMap();
      map.put("token", token);
      String checkTokenUrl = getUserCenterTokenURL(request);
      String url = HttpUtils.buildGetParam(checkTokenUrl, map);
      ResponseMsg respMsg = null;

      for(int i = 1; i <= 3; ++i) {
         try {
            String e = null;
            JSONObject result = HttpUtils.sendGet(url);
            if(result != null) {
               e = result.toString();
            }

            respMsg = getRespMsg(e);
            if(respMsg != null) {
               break;
            }

            logger.error("远程校验Token失败:{}", token);
         } catch (Exception var10) {
            logger.error("远程校验Token失败:{}" + token + ",msg:" + var10.getMessage(), var10);
         }

         try {
            Thread.sleep((long)(i * 1000));
         } catch (InterruptedException var9) {
            logger.error("Fail to sleep:" + var9.getLocalizedMessage());
         }
      }

      if(respMsg == null) {
         logger.error("远程校验Token失败：{}", token);
         return null;
      } else if(!respMsg.isOk()) {
         logger.error("远程校验Token失败:" + token + ",code:" + respMsg.getCode() + ",msg:" + respMsg.getMsg());
         return null;
      } else if(respMsg.getBody() == null) {
         logger.error("远程校验Token失败：{}", token);
         return null;
      } else {
         return (UserCenterLogin)respMsg.getBody();
      }
   }

   public static ResponseMsg getRespMsg(String resp) {
      if(resp == null) {
         return null;
      } else {
         try {
            JSONObject e = new JSONObject(resp);
            ResponseMsg msg = new ResponseMsg();
            msg.setCode(e.getString("code"));
            msg.setMsg(e.getString("msg"));
            JSONObject jsonObject2 = e.optJSONObject("body");
            if(jsonObject2 != null) {
               UserCenterLogin login = UserCenterLogin.convert(jsonObject2);
               msg.setBody(login);
            }

            return msg;
         } catch (JSONException var5) {
            logger.error("Fail to read as json:" + resp, var5);
            return null;
         }
      }
   }

   public static String createToken() throws Exception {
      return "PT-" + UUID.randomUUID().toString().replace("-", "").toUpperCase();
   }

   public static Object deserialize(byte[] objectData, Class tClass) throws IOException {
      if(objectData == null) {
         return null;
      } else {
         StopWatch watch = new StopWatch("deserialize");
         watch.start("json");
         Object ret = JsonUtil.readValue(new String(objectData, "UTF-8"), tClass);
         watch.stop();
         return ret;
      }
   }

   public static byte[] serialize(Serializable value) throws IOException {
      return JsonUtil.writeValueAsString(value).getBytes("UTF-8");
   }

   public static void cacheLocalValue(String token, String schema, String key, Object value) {
      SessionValueCacheManager manager = SessionValueCacheManager.getInstance();
      manager.addValue(token, schema, key, value);
   }

   public static Map getLocalCachedValues(String token, String schema) {
      SessionValueCacheManager manager = SessionValueCacheManager.getInstance();
      return manager.getTokenValues(token, schema);
   }

   public static String getLocalTokenCookieName(HttpServletRequest request) {
      int port = request.getServerPort();
      String cookieName = "PTOKEN" + port;
      return cookieName;
   }

   public static void saveTokenCookie(String token, HttpServletRequest request, HttpServletResponse response, int intervalInSeconds) {
      Cookie cookie = new Cookie(getLocalTokenCookieName(request), token);
      CookieKeys.addCookie(request, response, cookie, false, intervalInSeconds);
   }

   public static void removeTokenCookie(HttpServletRequest request, HttpServletResponse response) {
      String name = getLocalTokenCookieName(request);
      Cookie[] cks = request.getCookies();
      if(cks != null) {
         for(int i = 0; i < cks.length; ++i) {
            if(cks[i].getName().equals(name)) {
               cks[i].setMaxAge(0);
               cks[i].setPath("/");
               cks[i].setValue("");
               response.addCookie(cks[i]);
               logger.debug("removed cookie " + name);
               break;
            }
         }
      }

   }

}
